@@ -6,7 +6,7 @@ E-mail:3247119728@qq.com
 Company:湖南零檬信息技术有限公司
 ============================
 
-本模块主要是为了解决多线程运行unittest测试用例的问题
+本模块主要是为了解决unittest运行测试用例的生成HTML报告的测试问题，同时扩展了邮件发送等功能
 该模块预留了两个入口，
 
 注意点：
@@ -17,10 +17,11 @@ import os
 import unittest
 import time
 from concurrent.futures.thread import ThreadPoolExecutor
-
 from unittestreport.core.sendEmail import SendEmail
 from unittestreport.core.testResult import TestResult, ReRunResult
+from unittestreport.core.dingtalk import DingTalk
 from jinja2 import Environment, FileSystemLoader
+import copy
 
 
 class TestRunner():
@@ -59,7 +60,7 @@ class TestRunner():
         self.result = []
         self.starttime = time.time()
 
-    def classification_suite(self):
+    def __classification_suite(self):
         """
         将测试套件中的用例，根据用例类位单位，拆分成多个测试套件，打包成列表类型
         :return: list-->[suite,suite,suite.....]
@@ -74,60 +75,14 @@ class TestRunner():
                 else:
                     wrapper(item)
 
-        wrapper(self.suite)
+        wrapper(copy.deepcopy(self.suite))
         return suites_list
 
-    def classification_test_case(self):
+    def __get_reports(self):
         """
-        将测试套件中的用例进行拆分，保存到列表中
-        :return: list-->[case,case]
+        生成报告,返回测试汇中的结果
+        :return: 包含测试结果的字典
         """
-        test_list = []
-
-        def wrapper(suite):
-            for item in suite:
-                if isinstance(item, unittest.TestCase):
-                    test_list.append(item)
-                else:
-                    wrapper(item)
-
-        wrapper(self.suite)
-        return test_list
-
-    def run(self, thread_count=1):
-        """
-        支持多线程执行
-        注意点：如果多个测试类共用某一个全局变量，由于资源竞争可能会出现错误
-        :param thread_count:线程数量，默认位1
-        :return:
-        """
-        # 将测试套件按照用例类进行拆分
-        suites = self.classification_suite()
-        with ThreadPoolExecutor(max_workers=thread_count) as ts:
-            for i in suites:
-                res = TestResult()
-                self.result.append(res)
-                ts.submit(i.run, result=res).add_done_callback(res.stopTestRun)
-            ts.shutdown(wait=True)
-        self.get_reports()
-
-    def rerun_run(self, count=0, interval=2):
-        """
-        测试用例失败、错误重跑机制
-        :param count: 重跑次数，默认为0
-        :param interval: 重跑时间间隔，默认为2
-        :return:
-        """
-        res = ReRunResult(count=count, interval=interval)
-        self.result.append(res)
-        suites = self.classification_suite()
-        for case in suites:
-            case.run(res)
-        res.stopTestRun()
-        self.get_reports()
-
-    def get_reports(self):
-        """生成报告"""
         print("所有用例执行完毕，正在生成测试报告中......")
         # 汇总测试结果
         test_result = {
@@ -173,6 +128,49 @@ class TestRunner():
         self.email_conent = {"file": os.path.abspath(file_path),
                              "content": env.get_template('templates03.html').render(test_result)
                              }
+        self.test_result = test_result
+        return test_result
+
+    def __get_dingtalk_content(self):
+        """获取钉钉通知的内容"""
+        template_path = os.path.join(os.path.dirname(__file__), '../templates')
+        env = Environment(loader=FileSystemLoader(template_path))
+        res_text = env.get_template('dingtalk.md').render(self.test_result)
+        return res_text
+
+    def run(self, thread_count=1):
+        """
+        支持多线程执行
+        注意点：如果多个测试类共用某一个全局变量，由于资源竞争可能会出现错误
+        :param thread_count:线程数量，默认位1
+        :return:测试运行结果
+        """
+        # 将测试套件按照用例类进行拆分
+        suites = self.__classification_suite()
+        with ThreadPoolExecutor(max_workers=thread_count) as ts:
+            for i in suites:
+                res = TestResult()
+                self.result.append(res)
+                ts.submit(i.run, result=res).add_done_callback(res.stopTestRun)
+            ts.shutdown(wait=True)
+        res = self.__get_reports()
+        return res
+
+    def rerun_run(self, count=0, interval=2):
+        """
+        测试用例失败、错误重跑机制
+        :param count: 重跑次数，默认为0
+        :param interval: 重跑时间间隔，默认为2
+        :return: 测试运行结果
+        """
+        res = ReRunResult(count=count, interval=interval)
+        self.result.append(res)
+        suites = self.__classification_suite()
+        for case in suites:
+            case.run(res)
+        res.stopTestRun()
+        res = self.__get_reports()
+        return res
 
     def send_email(self, host, port, user, password, to_addrs, is_file=True):
         """
@@ -192,3 +190,53 @@ class TestRunner():
         content = self.email_conent["content"]
 
         sm.send_email(subject=self.title, content=content, filename=filename, to_addrs=to_addrs)
+
+    def get_except_info(self):
+        """
+        获取错误用例和失败用例的报错信息
+        :return:
+        """
+        except_info = []
+        for i in self.result:
+            for texts in i.failures:
+                t, content = texts
+                except_info.append("用例【{}】执行失败，失败信息如下：".format(t._testMethodDoc))
+                except_info.append(content)
+            for texts in i.errors:
+                t, content = texts
+                except_info.append("用例【{}】执行错误，错误信息如下：".format(t._testMethodDoc))
+                except_info.append(content)
+        except_str = "\n".join(except_info)
+        return except_str
+
+    def dingtalk_notice(self, url, key=None, secret=None, atMobiles=None, isatall=False, except_info=False):
+        """
+        钉钉通知
+        :param url: 钉钉机器人的Webhook地址
+        :param key: （非必传：str类型）如果钉钉机器人安全设置了关键字，则需要传入对应的关键字
+        :param secret:（非必传:str类型）如果钉钉机器人安全设置了签名，则需要传入对应的密钥
+        :param atMobiles: （非必传，list类型）发送通知钉钉中要@人的手机号列表，如：[137xxx,188xxx]
+        :param isatall: 是否@所有人，默认为False,设为True则会@所有人
+        :param except_info:是否发送未通过用例的详细信息，默认为False，设为True则会发送失败用例的详细信息
+        :return:
+        """
+        # 获取用例执行的结果,转换为md格式
+        res_text = self.__get_dingtalk_content()
+        # 判断是否需要发送错误用例的信息
+        if except_info:
+            res_text += '### 未通过用例详情：\n'
+            res_text += self.get_except_info()
+        # 构建通知内容
+        data = {
+            "msgtype": "markdown",
+            "markdown": {
+                "title": '{}({})'.format(self.title, key),
+                "text": res_text
+            },
+            "at": {
+                "atMobiles": atMobiles,
+                "isAtAll": isatall
+            }
+        }
+        ding = DingTalk(url=url, data=data, secret=secret)
+        ding.send_info()
